using System.Text;
using Tiger.Utils;

namespace Tiger.Communication.Messages.Types;

public class ClientMessage
{
    private readonly byte[] _packet;
    private int _readerIndex;
    
    public short? Header { get; }

    public ClientMessage(byte[] packet)
    {
        _packet = packet;
        _readerIndex = 0;
        Header = ReadInt16();
    }

    public short? ReadInt16()
    {
        var raw = ReadBytes(2);

        if (raw.Length != 2) return null;

        return ByteUtils.GetInt16(raw);
    }

    public int? ReadInt32()
    {
        var raw = ReadBytes(4);

        if (raw.Length != 4) return null;

        return ByteUtils.GetInt32(raw);
    }

    public bool? ReadBoolean()
    {
        if (_packet.Length - _readerIndex == 0) return null;
        
        var result = _packet[_readerIndex];
        _readerIndex++;
        return result == 1;
    }

    public string? ReadString()
    {
        var strlen = ReadInt16();

        if (strlen == null) return null;

        var raw = ReadBytes(strlen.Value);

        return raw.Length != strlen ? null : Encoding.UTF8.GetString(raw);
    }

    private byte[] ReadBytes(int amount)
    {
        if (_packet.Length - _readerIndex < amount)
        {
            return Array.Empty<byte>();
        }

        var data = new byte[amount];
        Array.Copy(_packet, _readerIndex, data, 0, amount);
        _readerIndex += amount;
        return data;
    }
}